@@ -2,6 +2,7 @@
 use strict;
 $| = 1;
 
+# $Id$
 use vars qw( $VERSION );
 $VERSION = '0.017';
 
@@ -23,17 +24,13 @@ use Getopt::Long;
 my %opt = (
     type         => undef,
     ddir         => undef,
-    to           => undef,
+    to           => undef,    #'smokers-reports@perl.org',
     cc           => undef,
     ccp5p_onfail => undef,
     from         => undef,
-    mserver      => undef,
-    msuser       => undef,
-    mspass       => undef,
     v            => undef,
 
     smokedb_url => undef,
-    ua_timeout  => undef,
 
     rptfile    => 'mktest.rpt',
     jsnfile    => 'mktest.jsn',
@@ -104,8 +101,7 @@ GetOptions(
     \%opt => qw(
         type|t=s
         ddir|d=s
-        to=s      cc=s      bcc=s     swcc=s swbcc=s
-        mserver=s msuser=s  mspass=s
+        to=s      cc=s      bcc=s
 
         ccp5p_onfail!
         v|verbose=i
@@ -161,15 +157,9 @@ if ($json && $opt{smokedb_url}) {
     require LWP::UserAgent;
     my $ua = LWP::UserAgent->new(
         agent => "Test::Smoke/$Test::Smoke::VERSION",
-        (
-            defined $opt{ua_timeout}
-                ? (timeout => $opt{ua_timeout})
-                : ()
-        ),
     );
     $opt{v} and print "Posting to SmokeDB ($opt{smokedb_url})\n";
-    my $response = eval {$ua->post($opt{smokedb_url}, {json => $json})};
-    print $@ if $@;
+    my $response = $ua->post($opt{smokedb_url}, {json => $json});
     $opt{v} and print $response->content;
 }
 
@@ -188,8 +178,8 @@ sub check_for_report {
     my $reporter = Test::Smoke::Reporter->new( $conf );
     if ( defined $reporter->{_outfile} ) {
         $reporter->write_to_file;
-
-        if (!-f $report) {
+    
+        unless ( -f $report ) {
             die "Hmmm... cannot find [$report]";
         }
         return 1;
@@ -201,25 +191,17 @@ sub check_for_report {
 
 sub check_for_json {
     my $jsnfile = File::Spec->catfile($opt{ddir}, $opt{jsnfile});
-    if (-f $jsnfile) {
-        $opt{v} and print "Found [$jsnfile]\n";
-        if (! $opt{report}) {
-            open my $jsn, '<', $jsnfile or die "Cannot open($jsnfile): $!";
-            my $json = do {local $/; <$jsn> };
-            close $jsn;
-            $opt{v} and print "Using JSON from '$jsnfile'\n";
-            return $json;
-        }
-        else {
-            $opt{v} and print "Not reusing existing JSON, regenerate\n";
-        }
-    }
-    else {
-        $opt{v} and print "No JSON found in [$opt{ddir}]\n";
+    if (! -f $jsnfile or $opt{report}) { 
+        my $reporter = Test::Smoke::Reporter->new($conf);
+        return $reporter->smokedb_data();
     }
 
-    my $reporter = Test::Smoke::Reporter->new($conf);
-    return $reporter->smokedb_data();
+    $opt{v} and print "Found [$jsnfile]\n";
+    local *JSON;
+    open JSON, '<', $jsnfile or die "Cannot open($jsnfile): $!";
+    my $json = do {local $/; <JSON> };
+    close JSON;
+    return $json;
 }
 
 =head1 SEE ALSO
