@@ -1,15 +1,19 @@
 package TestLib;
 use strict;
 
-# $Id: TestLib.pm 279 2003-07-28 19:48:46Z abeltje $
+# $Id: TestLib.pm 1053 2007-08-11 11:17:47Z abeltje $
 use vars qw( $VERSION @EXPORT );
-$VERSION = '0.02';
+$VERSION = '0.06';
+
+use Test::Smoke::Util qw( whereis );
 
 use base 'Exporter';
 @EXPORT = qw( 
-    &whereis 
+    &whereis
+    &find_a_patch
     &find_unzip &do_unzip
     &find_untargz &do_untargz
+    &manify_path
     &get_dir &get_file &put_file
     &rmtree &mkpath
 );
@@ -30,35 +34,32 @@ What is in here?
 
 =cut
 
+use Config;
 use File::Find;
-use File::Spec;
+use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs
+                              splitdir splitpath catpath);
 require File::Path;
 use Cwd;
 
-=item whereis( $prog )
+=item manify_path( $path )
 
-Try to find an executable instance of C<$prog> in $ENV{PATH}.
-
-Rreturns a full file-path (with extension) to it.
+Do a OS-specific split on the path, and join with '/' for MANIFEST
+format.
 
 =cut
 
-sub whereis {
-    my $prog = shift;
-    return undef unless $prog; # you shouldn't call it '0'!
+sub manify_path($) {
+    my $path = shift or return;
+    # There should be no volume on these file_paths
+    $path = File::Spec->canonpath( $path );
+    my( undef, $dirs, $file ) = File::Spec->splitpath( $path );
 
-    my $p_sep = $Config::Config{path_sep};
-    my @path = split /\Q$p_sep\E/, $ENV{PATH};
-    my @pext = split /\Q$p_sep\E/, $ENV{PATHEXT} || '';
-    unshift @pext, '';
+    my @subdirs = grep $_ && length $_ => File::Spec->splitdir( $dirs );
+    $^O eq 'VMS' and $file =~ s/\.$//;
 
-    foreach my $dir ( @path ) {
-        foreach my $ext ( @pext ) {
-            my $fname = File::Spec->catfile( $dir, "$prog$ext" );
-            return $fname if -x $fname;
-        }
-    }
-    return '';
+    push @subdirs, $file;
+
+    return join '/', @subdirs;
 }
 
 =item get_dir( $path )
@@ -69,13 +70,16 @@ Returns a list of filenames (no directory-names) in C<$path>.
 
 sub get_dir($) {
     my( $path ) = @_;
+    my $cwd = cwd();
+    chdir $path or die "Cannot chdir($path): $!";
     my @files;
     find sub {
         -f or return;
-        (my $name = $File::Find::name ) =~ s/^\Q$path\E//;
-        push @files, $name;
-    }, $path;
+        my $cname = File::Spec->canonpath( $File::Find::name );
+        push @files, $cname;
+    }, '.';
 
+    chdir $cwd or die "Cannot chdir($cwd) back: $!";
     return @files;
 }
 
@@ -145,6 +149,22 @@ This is B<< File::Path::mkpath() >>.
 
 sub mkpath { File::Path::mkpath( @_ ) }
 
+=item find_a_patch()
+
+Loop over some known names for gnu-patch and see if they know about --version.
+
+=cut
+
+sub find_a_patch {
+
+    my $patch_bin;
+    foreach my $patch (qw( gpatch npatch patch )) {
+        $patch_bin = whereis( $patch ) or next;
+        my $version = `$patch_bin --version`;
+        $? or return $patch_bin;
+    }
+}
+
 =item find_unzip()
 
 Check C<< wheris( 'gzip' ) >> or C<< eval{ require Compress::Zlib } >>.
@@ -179,7 +199,7 @@ sub do_unzip {
         require Compress::Zlib;
         my $unzipper = Compress::Zlib::gzopen( $uzfile, 'rb' ) or do {
             require Carp;
-            Carp::carp "Can't open '$uzfile': $Compress::Zlib::gzerrno";
+            Carp::carp( "Can't open '$uzfile': $Compress::Zlib::gzerrno" );
             return undef;
         };
 
@@ -188,7 +208,7 @@ sub do_unzip {
 
         unless ( $Compress::Zlib::gzerrno == Compress::Zlib::Z_STREAM_END() ) {
             require Carp;
-            Carp::carp "Error reading '$uzfile': $Compress::Zlib::gzerrno" ;
+            Carp::carp( "Error reading '$uzfile': $Compress::Zlib::gzerrno" );
         }
 
         $unzipper->gzclose;
@@ -264,6 +284,8 @@ sub do_untargz {
         $archive->extract( @files );
 
     } else { # assume command
+        $^O eq 'VMS' and return vms_untargz( $untgz, $tgzfile );
+
         my $command = sprintf $untgz, $tgzfile;
         $command .= " $tgzfile" if $command eq $untgz;
 
@@ -276,6 +298,42 @@ sub do_untargz {
     return 1;
 }
 
+=item vms_untargz( $untargz, $tgzfile )
+
+Gunzip and extract the archive in C<$tgzfile>.
+
+=cut
+
+sub vms_untargz {
+    my( $cmd, $file ) = @_;
+    my( $vol, $path, $fname ) = splitpath( $file );
+    my @parts = split /[.@#]/, $fname;
+    if ( @parts > 1 ) {
+        my $ext = ( pop @parts ) || '';
+        $fname = join( "_", @parts ) . ".$ext";
+    }
+    $file = catpath( $vol, $path, $fname );
+
+    my( $gzip_cmd, $tar_cmd ) = split /\s*\|\s*/, $cmd;
+    my $gzip = $gzip_cmd =~ /^(\S+)/ ? $1 : 'GZIP';
+    my $tar  = $tar_cmd  =~ /^(\S+)/
+        ? $1 : (whereis( 'vmstar' ) || whereis( 'tar' ) );
+
+    local *TMPCOM;
+    open TMPCOM, "> TS-UNTGZ.COM" or return 0;
+    print TMPCOM <<EO_UNTGZ; close TMPCOM or return 0;
+\$ define/user sys\$output TS-UNTGZ.TAR
+\$ $gzip "-cd" $file
+\$ $tar "-xf" TS-UNTGZ.TAR
+\$ delete TS-UNTGZ.TAR;*
+EO_UNTGZ
+
+    my $ret = system "\@TS-UNTGZ.COM";
+    1 while unlink "TS-UNTGZ.COM";
+
+    return ! $ret;
+}
+
 1;
 
 =back
